@@ -4,9 +4,10 @@
 # utils.c
 #
 # copyright (c) 2006, Carter T. Butts <buttsc@uci.edu>
-# Last Modified 6/6/09
+# Last Modified 12/13/20
 # Licensed under the GNU General Public License version 2 (June, 1991)
-# later.
+# or later.
+#
 # Portions taken from the NetStat library by Carter T. Butts (2002)
 #  (self-licensed under GPL)
 #
@@ -60,13 +61,13 @@ void logadd_R(double *lvals, int *n, double *lsum)
 {
   int i;
   
-  if(*n==1){
-    *lsum=lvals[0];
+  *lsum=lvals[0];  /*Initialize with the first value*/
+  if(*n==1){       /*If that's all there is, return*/
     return;
   }
-  *lsum=logspace_add(lvals[0],lvals[1]);
-  for(i=2;i<*n;i++)
-    *lsum=logspace_add(*lsum,lvals[i]);
+  for(i=1;i<*n;i++)  /*Otherwise, add the remaining entries*/
+    if(lvals[i]!=R_NegInf)   /*We can avoid issues/save time this way....*/
+      *lsum=logspace_add(*lsum,lvals[i]);
 }
 
 
@@ -521,6 +522,29 @@ return head;
 }
 
 
+element *pushCalloc(element *head, double val, void *dp)
+/*Adds element with value val to the stack, returning the head 
+pointer.  This function uses Calloc for memory allocation, and it must be
+manually deallocated with Free.  Do not use with stackdel et al!*/
+{
+element *newnode;
+
+/*Create the new entry*/
+newnode=(element *)R_alloc(1,sizeof(struct elementtype));
+
+newnode->val=val;   /*Set the element value*/
+newnode->dp=dp;
+
+/*Set the next pointer equal to the current first entry (if any)*/
+newnode->next=head;
+
+/*Place the new node at the head of the stack*/
+head=newnode;
+
+return head;
+}
+
+
 long int stacklen(element *head)
 /*Returns the length of the stack pointed to by head*/
 {
